package com.mdd.admin.aop.aspect;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.mdd.admin.LikeAdminThreadLocal;
import com.mdd.admin.aop.Log;
import com.mdd.common.entity.system.SystemLogOperate;
import com.mdd.common.mapper.system.SystemLogOperateMapper;
import com.mdd.common.util.IpUtils;
import com.mdd.common.util.JwtUtils;
import com.mdd.common.util.RequestUtils;
import io.jsonwebtoken.Claims;
import java.lang.reflect.Method;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.support.StandardMultipartHttpServletRequest;

@Aspect
@Component
public class LogAspect {
   @Resource
   SystemLogOperateMapper systemLogOperateMapper;
   private static final Logger log = LoggerFactory.getLogger(LogAspect.class);
   private static final ThreadLocal<Long> threadLocal = new ThreadLocal();
   private final ExecutorService executor = Executors.newSingleThreadExecutor();

   @Pointcut("@annotation(com.mdd.admin.aop.Log)")
   private void pointCutMethodController() {
   }

   @Around("pointCutMethodController()")
   public Object doAroundService(ProceedingJoinPoint joinPoint) throws Throwable {
      threadLocal.set(System.currentTimeMillis());
      Object result = joinPoint.proceed();
      this.recordLog(joinPoint, result, (Exception)null);
      return result;
   }

   @AfterThrowing(
      value = "@annotation(controllerLog)",
      throwing = "e"
   )
   public void doAfterThrowing(JoinPoint joinPoint, Log controllerLog, Exception e) {
      this.recordLog(joinPoint, (Object)null, e);
   }

   private void recordLog(Object joinPointObj, Object result, Exception e) {
      try {
         long beginTime = (Long)threadLocal.get();
         long endTime = System.currentTimeMillis();
         ServletRequestAttributes requestAttributes = (ServletRequestAttributes)RequestContextHolder.getRequestAttributes();
         if (requestAttributes != null) {
            HttpServletRequest request = requestAttributes.getRequest();
            ProceedingJoinPoint joinPoint = (ProceedingJoinPoint)joinPointObj;
            MethodSignature signature = (MethodSignature)joinPoint.getSignature();
            Method method = signature.getMethod();
            Log logAnnotation = (Log)method.getAnnotation(Log.class);
            Integer adminId = null;
            if (logAnnotation.isClient() == IsClient.TRUE) {
               try {
                  Claims claims = JwtUtils.parseToken(request.getHeader("Authorization"));
                  adminId = (Integer)claims.get("orderId", Integer.class);
               } catch (Exception var27) {
               }
            } else {
               try {
                  adminId = LikeAdminThreadLocal.getAdminId();
               } catch (Exception var26) {
               }
            }

            String className = joinPoint.getTarget().getClass().getName();
            String methodName = joinPoint.getSignature().getName();
            String queryString = request.getQueryString();
            Object[] args = joinPoint.getArgs();
            String params = "";
            if (args.length > 0) {
               if ("POST".equals(request.getMethod())) {
                  if (RequestType.File.equals(logAnnotation.requestType())) {
                     StandardMultipartHttpServletRequest standardMultipartHttpServletRequest = (StandardMultipartHttpServletRequest)args[0];
                     params = (String)standardMultipartHttpServletRequest.getMultiFileMap().values().stream().map((m) -> {
                        return (String)m.stream().map(MultipartFile::getOriginalFilename).collect(Collectors.joining(","));
                     }).collect(Collectors.joining(","));
                  } else {
                     params = JSON.toJSONString(args);
                  }
               } else if ("GET".equals(request.getMethod())) {
                  params = queryString;
               }
            }

            String error = "";
            int status = 1;
            if (e != null) {
               error = e.getMessage();
               status = 2;
            }

            SystemLogOperate model = new SystemLogOperate();
            model.setAdminId(adminId);
            model.setIsClient(logAnnotation.isClient() == IsClient.TRUE ? 1 : 0);
            model.setTitle(logAnnotation.title());
            model.setIp(IpUtils.getIpAddress());
            model.setType(request.getMethod());
            model.setMethod(className + "." + methodName + "()");
            model.setUrl(RequestUtils.route());
            model.setArgs(params);
            String res = "";
            if (result != null) {
               try {
                  res = JSONObject.toJSONString(result, new JSONWriter.Feature[0]);
               } catch (Exception var25) {
               }
            }

            model.setResult(res);
            model.setError(error);
            model.setStatus(Integer.valueOf(status));
            model.setStartTime(beginTime / 1000L);
            model.setEndTime(endTime / 1000L);
            model.setTaskTime(endTime - beginTime);
            model.setCreateTime(System.currentTimeMillis() / 1000L);
            this.executor.submit(() -> {
               this.systemLogOperateMapper.insert(model);
            });
         }
      } catch (Exception var28) {
         Exception ex = var28;
         log.error("异常信息:{}", ex.getMessage());
      }

   }
}
